// TrayIcon.cpp: implementation of the CTrayIcon class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TrayIcon.h"
#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTrayIcon::CTrayIcon()
{
	m_hInstance = NULL;
	memset(&m_tnd, 0, sizeof(m_tnd));
	m_Animate_Number = 0;
}

CTrayIcon::CTrayIcon(HINSTANCE hInstance)
{
	memset(&m_tnd, 0, sizeof(m_tnd));
	m_hInstance = hInstance;
	m_Animate_Number = 0;
}

CTrayIcon::CTrayIcon(HWND hWnd, UINT uCallbackMessage, LPCTSTR szToolTip, 
					 HICON icon, UINT uID, HINSTANCE hInstance)
{
	m_hInstance = hInstance;
	m_Animate_Number = 0;
	Create(hWnd, uCallbackMessage, szToolTip, icon, uID);
}

CTrayIcon::~CTrayIcon()
{
	RemoveIcon();
}

BOOL CTrayIcon::Create(HWND hWnd, UINT uCallbackMessage, LPCTSTR szToolTip, 
					   HICON icon, UINT uID)
{
	//Make sure we avoid conflict with other messages
	_ASSERT(uCallbackMessage != WM_USER);

	//Tray only supports tooltip text up to 64 characters
	_ASSERT(_tcslen(szToolTip) <= 64);

	// NOTIFYICONDATA ü  
	m_tnd.cbSize = NOTIFYICONDATA_V1_SIZE;
	m_tnd.hWnd	 = hWnd;
	m_tnd.uID	 = uID;
	m_tnd.hIcon  = icon;
	m_tnd.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
	m_tnd.uCallbackMessage = uCallbackMessage;
	m_tnd.uVersion = NOTIFYICON_VERSION;

	_tcscpy_s (m_tnd.szTip, sizeof(m_tnd.szTip)/sizeof(m_tnd.szTip[0]), szToolTip);
	// Ʈ  ߰
	return Shell_NotifyIcon(NIM_ADD, &m_tnd);
}

/////////////////////////////////////////////////////////////////////////////
// CTrayIcon icon manipulation

void CTrayIcon::RemoveIcon()
{
	m_tnd.uFlags = 0;
    Shell_NotifyIcon(NIM_DELETE, &m_tnd);
}

BOOL CTrayIcon::SetIcon(HICON hIcon)
{
	m_tnd.uFlags = NIF_ICON;
	m_tnd.hIcon = hIcon;

	return Shell_NotifyIcon(NIM_MODIFY, &m_tnd);
}

BOOL CTrayIcon::SetIcon(LPCTSTR lpszIconName)
{
	HICON hIcon = ::LoadIcon(m_hInstance, lpszIconName);

	return SetIcon(hIcon);
}

BOOL CTrayIcon::SetIcon(UINT nIDResource)
{
	HICON hIcon = ::LoadIcon(m_hInstance, MAKEINTRESOURCE(nIDResource));

	return SetIcon(hIcon);
}

BOOL CTrayIcon::SetStandardIcon(LPCTSTR lpIconName)
{
	HICON hIcon = LoadIcon(NULL, lpIconName);

	return SetIcon(hIcon);
}

BOOL CTrayIcon::SetStandardIcon(UINT nIDResource)
{
	HICON hIcon = LoadIcon(NULL, MAKEINTRESOURCE(nIDResource));

	return SetIcon(hIcon);
}

HICON CTrayIcon::GetIcon() const
{
	HICON hIcon = NULL;
	hIcon = m_tnd.hIcon;

	return hIcon;
}

/////////////////////////////////////////////////////////////////////////////
// CTrayIcon tooltip text manipulation

BOOL CTrayIcon::SetTooltipText(LPCTSTR pszTip)
{
	m_tnd.uFlags = NIF_TIP;
	lstrcpy(m_tnd.szTip, pszTip);

	return Shell_NotifyIcon(NIM_MODIFY, &m_tnd);
}

BOOL CTrayIcon::SetTooltipText(UINT nID)
{
	ATL::CString strText;
	//VERIFY(strText.LoadString(nID));

	return SetTooltipText(strText);
}

ATL::CString CTrayIcon::GetTooltipText()
{
	ATL::CString strText;
	strText = m_tnd.szTip;

	return strText;
}

/////////////////////////////////////////////////////////////////////////////
// CTrayIcon notification window stuff

BOOL CTrayIcon::SetNotificationWnd(HWND hWnd)
{
	//Make sure Notification window is valid
	_ASSERT(::IsWindow(hWnd));

	m_tnd.hWnd = hWnd;
	m_tnd.uFlags = 0;

	return Shell_NotifyIcon(NIM_MODIFY, &m_tnd);
}

HWND CTrayIcon::GetNotificationWnd() const
{
	return m_tnd.hWnd;
}

/////////////////////////////////////////////////////////////////////////////
// CTrayIcon implentation of OnTrayNotification


int CTrayIcon::Animate()
{
	m_Animate_Number = (m_Animate_Number+1)%m_IconList.size();
	SetIcon(m_IconList[m_Animate_Number]);
	return 0;
}

void CTrayIcon::PopupBalloon(const TCHAR *text, BOOL bShow)
{
	m_tnd.cbSize = NOTIFYICONDATA_V2_SIZE;
	m_tnd.uFlags = NIF_INFO;

	m_tnd.szTip[0] = 0;
	if (bShow)
		_tcscpy_s(m_tnd.szInfo, sizeof(m_tnd.szInfo)/sizeof(m_tnd.szInfo[0]), text );
	else
		m_tnd.szInfo[0] = '\0';

	m_tnd.uTimeout = 0;
	m_tnd.dwInfoFlags = NIIF_INFO;
	
	Shell_NotifyIcon(NIM_MODIFY,&m_tnd);
}

